#!/usr/bin/env python

"""Tests for `swesarr_pytools` package."""

#
# import unittest
#
# from swesarr_pytools import swesarr_pytools
#
#
# class TestSwesarr_pytools(unittest.TestCase):
#     """Tests for `swesarr_pytools` package."""
#
#     def setUp(self):
#         """Set up test fixtures, if any."""
#
#     def tearDown(self):
#         """Tear down test fixtures, if any."""
#
#     def test_000_something(self):
#         """Test something."""
